@@ -18,6 +18,7 @@ import socket
 import sys
 import traceback
 import time
+import glob
 import base64
 from collections import Mapping, Set, Sequence, OrderedDict
 
@@ -26,6 +27,7 @@ from future.utils import with_metaclass
 from dataanalysis import hashtools
 from dataanalysis import jsonify
 from dataanalysis.bcolors import render
+import dataanalysis.bcolors as bcolors
 from dataanalysis.caches import cache_core
 
 from dataanalysis.analysisfactory import AnalysisFactory
@@ -308,6 +310,8 @@ class DataAnalysisIdentity(object):
     def serialize(self):
         result=self.__dict__
 
+        return result
+
         serialized_assumptions=[]
         for assumption in result['assumptions']:
             if isinstance(assumption,tuple) and isinstance(assumption[1],dict):
@@ -420,14 +424,20 @@ class DataAnalysis(with_metaclass(decorate_all_methods, object)):
         return name
 
     def get_identity(self):
-        log("assembling portable identity")
+        log("assembling portable identity", level="top")
         log("object assumptions:",self.assumptions)
         log("factory assumptions:", self.factory.cache_assumptions)
+
+        assumptions = []
+        for a in [a.serialize() for a in (self.factory.factory_assumptions_stacked + self.assumptions)]:
+            if len(assumptions) == 0 or assumptions[-1] != a:
+                assumptions.append(a)
+
         return DataAnalysisIdentity(
             factory_name=self.get_factory_name(),
             full_name=self.get_version(),
             modules = self.factory.get_module_description(),
-            assumptions=[a.serialize() for a in (self.factory.factory_assumptions_stacked + self.assumptions)],
+            assumptions=assumptions,
             expected_hashe=self.expected_hashe,
         )
 
@@ -677,7 +687,7 @@ class DataAnalysis(with_metaclass(decorate_all_methods, object)):
         else:
             return ""
 
-    def get_version(self):
+    def get_version(self, for_repr=False):
         ss="_".join(["%s:%s"%(a,repr(getattr(self,a))) for a in self._da_settings]) if self._da_settings is not None else ""
         v=self.get_signature()+"."+self.version+("."+ss if ss!="" else "")
         #if hasattr(self,'timestamp'):
@@ -712,29 +722,30 @@ class DataAnalysis(with_metaclass(decorate_all_methods, object)):
         if isinstance(content,dict):
             for a,b in list(content.items()):
                 if isinstance(b,DataFile):
-                    dest_unique=b.path+"."+self.cache.hashe2signature(hashe) # will it always?
-                    b._da_unique_local_path=dest_unique
-                    shutil.copyfile(b.path,dest_unique)
-                    log("post-processing DataFile",b,"as",b._da_unique_local_path,log='datafile')
+                    dest_unique = b.path+"."+self.cache.hashe2signature(hashe) # will it always?
+                    b._da_unique_local_path = dest_unique
+
+                    if os.path.exists(b.path):
+                        shutil.copyfile(b.path, dest_unique)
+                        log("post-processing DataFile",b,"as",b._da_unique_local_path,log='datafile')
+                    else:
+                        log("problem post-processing output file", b.path, log='datafile')
+                        raise Exception("problem post-processing output file " + b.path)
 
     def store_cache(self,fih):
         """
         store output 
         """
 
-        log(render("{MAGENTA}storing in cache{/} object: %s, next cache %s"%(repr(self), repr(self.cache))),level="top")
+        log(render("{MAGENTA}object storing in cache{/}: %s, cache %s"%(repr(self), repr(self.cache))),level="top")
         log("hashe:",fih)
 
-        #c=MemCacheLocal.store(fih,self.export_data())
-        #log(render("{MAGENTA}this is non-cached analysis, reduced caching: only transient{/}"))
-
         log("transient cache", TransientCacheInstance, "storing", self)
         TransientCacheInstance.store(fih,self)
 
-        log("further cache", self.cache, "storing", self, level="top")
+        log(render("{MAGENTA}first non-transient cache{/}"), self.cache, "storing", self, level="top")
         self.cache.store(fih,self)
 
-        #c=MemCacheLocal.store(oh,self.export_data())
 
     def post_restore(self):
         pass
@@ -742,7 +753,7 @@ class DataAnalysis(with_metaclass(decorate_all_methods, object)):
     _da_cache_retrieve_requests=None
 
     def retrieve_cache(self,fih,rc=None):
-        log(render("{CYAN}requested cache{/} for"), repr(self), level='top')
+        log(render("{CYAN}object requesting cache{/} for"), repr(self), " requested by "+(" ".join(self._da_requested_by)), level='top')
 
 
         if self._da_cache_retrieve_requests is None:
@@ -790,7 +801,7 @@ class DataAnalysis(with_metaclass(decorate_all_methods, object)):
         # discover through different caches
         #c=MemCacheLocal.find(fih)
 
-        log("searching for cache starting from",self.cache, level='top')
+        log(render("{CYAN}searching for cache starting from{/}"),self.cache, level='top')
         r=self.cache.restore(fih,self,rc)
         log("cache",self.cache,"returns",r, level='top')
 
@@ -1283,6 +1294,7 @@ class DataAnalysis(with_metaclass(decorate_all_methods, object)):
 
 
     def process(self,process_function=None,restore_rules=None,restore_config=None,requested_by=None,**extra):
+        log("stacked factory assumptions:", self.factory.factory_assumptions_stacked, level="assumptions")
         log(render("{BLUE}PROCESS{/} "+repr(self)))
 
         if requested_by is None:
@@ -1984,7 +1996,17 @@ class DataFile(DataAnalysis):
                 raise Exception("unable to construct full path!")
 
     def open(self, f="r"):
-        return gzip.open(self.cached_path) if hasattr(self,'cached_path') else open(self.path, f)
+        if hasattr(self,'cached_path'):
+            if os.path.exists(self.cached_path):
+                return gzip.open(self.cached_path)
+            else:
+                m = f"\033[31mERROR\033[0m: object {self} has cached path: {self.cached_path} but it does not exist\n"
+                m += f"\033[31mERROR\033[0m: cwd {os.getcwd()}\n"
+                m += f"\033[31mERROR\033[0m: have here {glob.glob('*')}"
+                log(m, level="top")
+                raise RuntimeError(m)
+        else:
+            return open(self.path, f)
 
     def __repr__(self):
         return "[DataFile:%s]"%(self.path if hasattr(self,'path') else 'undefined')
@@ -1995,10 +2017,10 @@ class DataFile(DataAnalysis):
 
         if self.size<100e3:
             try:
-                return json.load(self.open())
+                return json.load(self.open("rt"))
             except:    
                 try:
-                    content=self.open().read()
+                    content=self.open("rt").read()
                 except UnicodeDecodeError:
                     content=self.open("rb").read().decode('utf-8', 'ignore') #??
 
@@ -2009,11 +2031,10 @@ class DataFile(DataAnalysis):
                 log("can not interpret as fits:",e)
             
             try:
-                json.dumps(content)
+                json.dumps(content) # just to try
                 return content
-            except:
-                content=self.open().read()
-                return str(self)+" can not encode "+str(self.size)+" fits error "+str(e)
+            except Exception as e:
+                return str(self)+" can not encode to json "+str(self.size)+" fits error "+repr(e)
         else:
             return str(self)+" too big "+str(self.size)
 
